<?php

declare(strict_types=1);

namespace Pinpoint\TaxSwitch\Plugin\Magento\Checkout\CustomerData;

use Magento\Checkout\CustomerData\Cart;
use Magento\Checkout\Model\Session;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Quote\Model\Quote\Item;
use Magento\Weee\Block\Item\Price\Renderer;
use Pinpoint\TaxSwitch\Api\ConfigHelperInterface;

/**
 * This class intercepts methods from \Magento\Checkout\CustomerData\Cart
 * @see \Magento\Checkout\CustomerData\Cart
 */
class CartPlugin
{
    /**
     * @var string
     */
    protected const ITEM_PRICE_TEMPLATE = 'Pinpoint_TaxSwitch::checkout/cart/item/price/sidebar.phtml';

    /**
     * @var \Pinpoint\TaxSwitch\Api\ConfigHelperInterface
     */
    private $configHelper;

    /**
     * @var \Magento\Tax\Block\Item\Price\Renderer
     */
    private $renderer;

    /**
     * @var
     */
    private $quote;

    /**
     * @var \Magento\Checkout\Model\Session
     */
    private $session;

    /**
     * @param \Pinpoint\TaxSwitch\Api\ConfigHelperInterface $configHelper
     * @param \Magento\Checkout\Model\Session               $session
     * @param \Magento\Weee\Block\Item\Price\Renderer       $renderer
     */
    public function __construct(
        ConfigHelperInterface $configHelper,
        Session $session,
        Renderer $renderer
    ) {
        $this->configHelper = $configHelper;
        $this->session = $session;
        $this->renderer = $renderer;
    }

    /**
     * @param Cart  $subject
     * @param array $result
     *
     * @return array
     */
    public function afterGetSectionData(Cart $subject, array $result): array
    {
        if (!$this->configHelper->isEnabled() || !is_array($result['items']) || !$this->getQuote()) {
            return $result;
        }

        foreach ($result['items'] as &$item) {
            $quoteItem = $this->getQuoteItemById((int) $item['item_id']);

            if (!$quoteItem) {
                continue;
            }

            $this->renderer->setItem($quoteItem);
            $this->renderer->setTemplate(self::ITEM_PRICE_TEMPLATE);
            $item['product_price'] = $this->renderer->toHtml();
        }

        return $result;
    }

    /**
     * @return \Magento\Quote\Api\Data\CartInterface
     */
    private function getQuote(): \Magento\Quote\Api\Data\CartInterface
    {
        if (null === $this->quote) {
            try {
                $this->quote = $this->session->getQuote();
            } catch (NoSuchEntityException|LocalizedException $e) {
                $this->quote = null;
            }
        }

        return $this->quote;
    }

    /**
     * @param int $itemId
     *
     * @return \Magento\Quote\Model\Quote\Item|null
     */
    private function getQuoteItemById(int $itemId): ?Item
    {
        $items = $this->getQuote()->getAllVisibleItems();

        foreach ($items as $item) {
            if ((int) $item->getItemId() === $itemId) {
                return $item;
            }
        }

        return null;
    }
}
